#!/usr/bin/env node

var http = require('http');
var _ = require('lodash');
var express = require('express');
var fs = require('fs');
var path = require('path');
var util = require('util');

var program = require('commander');

function collect(val, memo) {
  if(val && val.indexOf('.') != 0) val = "." + val;
  memo.push(val);
  return memo;
}

program
  .option('-p, --port <port>', 'Port to run the file-browser. Default value is 8088')
  .option('-e, --exclude <exclude>', 'File extensions to exclude. To exclude multiple extension pass -e multiple times. e.g. ( -e .js -e .cs -e .swp) ', collect, [])
  .option('-d, --dir <dir>', 'Home directory', collect, [])
  .parse(process.argv);

var app = express();
var dir =  process.cwd();

//console.log("file-browser: CWD=["+dir+"]");
//console.log("file-browser: JSON.stringify(dir)=["+JSON.stringify(dir)+"]");
//console.log("file-browser: JSON.stringify(program.dir)=["+JSON.stringify(program.dir)+"]");

/**/
//console.log("file-browser: begin dir analysis: program.dir=["+program.dir+"]");
if(!program.dir){
	//console.log("file-browser: NO other user dir specified, using ["+dir+"]");
} else {
	var foo = program.dir.toString().substr(0,1)=="." ? program.dir.toString().substr(1) : program.dir.toString();  //fix up program.dir, seems to have a "." prepended
	dir=foo;
} 

/**************************/
// Add headers
app.use(function (req, res, next) {

    console.log('filebrowser: index.js: app.use adjusting Acess-Control-Allow-Origin header');
    console.log("req.originalUrl = ["+req.originalUrl+"]");

    // Website you wish to allow to connect
    res.setHeader('Access-Control-Allow-Origin', 'http://localhost:8000');

    // Request methods you wish to allow
    res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');

    // Request headers you wish to allow
    res.setHeader('Access-Control-Allow-Headers', 'X-Requested-With,content-type');

    // Set to true if you need the website to include cookies in the requests sent
    // to the API (e.g. in case you use sessions)
    res.setHeader('Access-Control-Allow-Credentials', true);

    // Pass to next layer of middleware
    next();
});
/**************************/



app.use(express.static(dir)); //app public directory

app.use(express.static(__dirname)); //module directory
var server = http.createServer(app);

if(!program.port) program.port = 8088;

server.listen(program.port);
console.log("Please open the link in your browser http://<YOUR-IP>:" + program.port);

app.get('/files', function(req, res) {
 var currentDir =  dir;
 var query = req.query.path || '';
 if (query) currentDir = path.join(dir, query);
 console.log("browsing ", currentDir);
 fs.readdir(currentDir, function (err, files) {
     if (err) {
        throw err;
      }
      var data = [];
      files
      .filter(function (file) {
          return true;
      }).forEach(function (file) {
        try {
                //console.log("processing ", file);
                var isDirectory = fs.statSync(path.join(currentDir,file)).isDirectory();
                if (isDirectory) {
                  data.push({ Name : file, IsDirectory: true, Path : path.join(query, file)  });
                } else {
                  var ext = path.extname(file);
                  if(program.exclude && _.contains(program.exclude, ext)) {
                    console.log("excluding file ", file);
                    return;
                  }       
                  data.push({ Name : file, Ext : ext, IsDirectory: false, Path : path.join(query, file) });
                }

        } catch(e) {
          console.log(e); 
        }        
        
      });
      data = _.sortBy(data, function(f) { return f.Name });
      res.json(data);
  });
});

app.get('/lib/template.html', function(req, res) {
 console.log("filebrowser: route /lib/template.html");
 //res.redirect('lib/template.html'); 
 next();
});

app.get('/', function(req, res) {
 res.redirect('lib/template.html'); 
});
